@@ -4,7 +4,7 @@ use warnings;
 use 5.010; # state
 
 # ABSTRACT: turn on Unicode - all of it
-our $VERSION = '0.020'; # VERSION
+our $VERSION = '0.022'; # VERSION
 
 #pod =head1 SYNOPSIS
 #pod
@@ -139,14 +139,14 @@ $Carp::Internal{$current_package}++; # To get warnings reported at correct calle
 #pod I<always> enforced.
 #pod
 #pod Important: Only controls the handling of decoding errors in C<glob>,
-#pod C<readdir>, C<readlink>.
+#pod C<readdir>, and C<readlink>.
 #pod
 #pod =cut
 
 use Encode ();
 use PerlIO::utf8_strict;
 
-our $UTF8_CHECK = Encode::FB_CROAK; # Die on encoding errors
+our $UTF8_CHECK = Encode::FB_CROAK | Encode::LEAVE_SRC; # Die on encoding errors
 
 # UTF-8 Encoding object
 my $_UTF8 = Encode::find_encoding('UTF-8');
@@ -220,12 +220,12 @@ sub unimport { ## no critic (Subroutines::ProhibitBuiltinHomonyms)
 
 sub _utf8_readdir(*) { ## no critic (Subroutines::ProhibitSubroutinePrototypes)
     my $pre_handle = shift;
-    $UTF8_CHECK |= Encode::LEAVE_SRC if $UTF8_CHECK; # Enforce LEAVE_SRC
-    my $handle = ref($pre_handle) ? $pre_handle : qualify_to_ref($pre_handle ? $_UTF8->encode($pre_handle, $UTF8_CHECK) : $pre_handle, caller);
     my $hints = (caller 0)[10];
+   my $handle = ref($pre_handle) ? $pre_handle : qualify_to_ref($pre_handle, caller);
     if (not $hints->{'utf8::all'}) {
         return CORE::readdir($handle);
     } else {
+        $UTF8_CHECK |= Encode::LEAVE_SRC if $UTF8_CHECK; # Enforce LEAVE_SRC
         if (wantarray) {
             return map { $_ ? $_UTF8->decode($_, $UTF8_CHECK) : $_ } CORE::readdir($handle);
         } else {
@@ -291,7 +291,7 @@ utf8::all - turn on Unicode - all of it
 
 =head1 VERSION
 
-version 0.020
+version 0.022
 
 =head1 SYNOPSIS
 
@@ -392,7 +392,7 @@ ignore them. Please see L<Encode> for details. Note: C<Encode::LEAVE_SRC> is
 I<always> enforced.
 
 Important: Only controls the handling of decoding errors in C<glob>,
-C<readdir>, C<readlink>.
+C<readdir>, and C<readlink>.
 
 =head1 INTERACTION WITH AUTODIE
 
@@ -447,7 +447,7 @@ Mike Doherty <doherty@cpan.org>
 
 =item *
 
-Hayo Baan <hayobaan@cpan.org>
+Hayo Baan <info@hayobaan.com>
 
 =back
 
